@@ -5,7 +5,7 @@
 ;; Maintainer: Antoine R. Dumont (@ardumont)
 ;; URL: http://github.com/ardumont/markdown-toc
 ;; Created: 24th May 2014
-;; Version: 0.1.4
+;; Version: 0.1.5
 ;; Keywords: markdown, toc, tools,
 ;; Package-Requires: ((markdown-mode "2.1") (dash "2.11.0") (s "1.9.0"))
 
@@ -62,7 +62,7 @@
 (require 'dash)
 (require 'markdown-mode)
 
-(defconst markdown-toc--toc-version "0.1.4" "Current version installed.")
+(defconst markdown-toc--toc-version "0.1.5" "Current version installed.")
 
 (defgroup markdown-toc nil
   "A simple TOC generator for markdown file."
@@ -142,7 +142,7 @@ it to the TOC structure."
   "Given LEVEL-TITLE-TOC-LIST, a list of pair level, title, return a TOC string."
   (->> level-title-toc-list
        markdown--count-duplicate-titles
-       (--map (let ((nb-spaces (* 4 (car it)))
+       (--map (let ((nb-spaces (* markdown-toc-indentation-space (car it)))
                     (title     (car (cdr it)))
                     (count     (car (cdr (cdr it)))))
                 (format "%s%s %s"
@@ -216,6 +216,9 @@ Return the end position if it exists, nil otherwise."
           toc
           markdown-toc-header-toc-end))
 
+(defcustom markdown-toc-indentation-space 4
+  "Let the user decide the indentation level.")
+
 (defcustom markdown-toc-user-toc-structure-manipulation-fn
   (lambda (toc-structure) toc-structure)
   "User crafted function to manipulate toc-structure as user sees fit.
@@ -261,6 +264,8 @@ If called interactively with prefix arg REPLACE-TOC-P, replaces previous TOC."
          markdown-toc--generate-toc
          insert)))
 
+(defalias 'markdown-toc/generate-toc 'markdown-toc-generate-toc)
+
 ;;;###autoload
 (defun markdown-toc-generate-or-refresh-toc ()
   "Generate a TOC for markdown file at current point or refreshes an already generated TOC."
@@ -281,7 +286,43 @@ If called interactively with prefix arg REPLACE-TOC-P, replaces previous TOC."
   (save-excursion
     (markdown-toc--delete-toc t)))
 
-(defalias 'markdown-toc/generate-toc 'markdown-toc-generate-toc)
+(defun markdown-toc--read-title-out-of-link (link)
+  "Extract the link title out of a markdown LINK title.
+This assumes no funky stuff in the markdown link format ` - [<title>](...) `  "
+  (->> link
+       s-trim
+       (s-chop-prefix "- [")
+       (s-split "]")
+       car))
+
+(defun markdown-toc--title-level (link)
+  "Determine the markdown title LINK out of its indentation.
+If misindented or not prefixed by `-`, it's considered not a link
+and returns nil. Otherwise, returns the level number."
+  (when (s-prefix? "-" (-> link s-trim)) ;; if not, it's not a link title
+    (let ((indent (->> link
+                       (s-split "-")
+                       car  ;; first string contains a string with empty spaces
+                       ;; which should be a multiple of
+                       ;; `markdown-toc-indentation-space`
+                       length)))
+      (when (zerop (% indent markdown-toc-indentation-space))
+        (+ 1 (/ indent markdown-toc-indentation-space))))))
+
+;;;###autoload
+(defun markdown-toc-follow-link-at-point ()
+  "On a given toc link, navigate to the current markdown header.
+If the toc is misindented (according to markdown-toc-indentation-space`)
+or if not on a toc link, this does nothing.
+"
+  (interactive)
+  (let* ((full-title (buffer-substring-no-properties (point-at-bol) (point-at-eol)))
+         (level (markdown-toc--title-level full-title)))
+    (if level ;; nil if misindented or not on a title
+        (let ((title (markdown-toc--read-title-out-of-link full-title)))
+          (goto-char (point-min))
+          (search-forward-regexp (format "%s %s" (s-repeat level "#") title)))
+      (message "markdown-toc: Not on a link (or misindented), nothing to do"))))
 
 (defun markdown-toc--bug-report ()
   "Compute the bug report for the user to include."
@@ -308,5 +349,30 @@ opens new issue in markdown-toc's github tracker."
     (browse-url "https://github.com/ardumont/markdown-toc/issues/new"))
   (markdown-toc-log-msg (list (markdown-toc--bug-report))))
 
+(defvar markdown-toc-mode-map nil "Default Bindings map for markdown-toc mode.")
+
+(setq markdown-toc-mode-map
+      (let ((map (make-sparse-keymap)))
+        (define-key map (kbd "C-c m .") 'markdown-toc-follow-link-at-point)
+        (define-key map (kbd "C-c m t") 'markdown-toc-generate-or-refresh-toc)
+        (define-key map (kbd "C-c m d") 'markdown-toc-delete-toc)
+        (define-key map (kbd "C-c m v") 'markdown-toc-version)
+        map))
+
+;;;###autoload
+(define-minor-mode markdown-toc-mode
+  "Functionality for generating toc in markdown file.
+With no argument, the mode is toggled on/off.
+Non-nil argument turns mode on.
+Nil argument turns mode off.
+
+Commands:
+\\{markdown-toc-mode-map}"
+
+  :init-value nil
+  :lighter " mt"
+  :group 'markdown-toc
+  :keymap markdown-toc-mode-map)
+
 (provide 'markdown-toc)
 ;;; markdown-toc.el ends here
